@@ -41,7 +41,7 @@ GetFootprintData <- function(
   positionEnrichment <- GetAssayData(
     object = object,
     assay = assay,
-    slot = "positionEnrichment"
+    layer = "positionEnrichment"
   )
   obj.groups <- GetGroups(
     object = object,
@@ -170,7 +170,7 @@ Footprint.ChromatinAssay <- function(
   }
   if (compute.expected) {
     # check that bias is computed
-    bias <- GetAssayData(object = object, slot = "bias")
+    bias <- GetAssayData(object = object, layer = "bias")
     if (is.null(x = bias)) {
       if (verbose) {
         message("Computing Tn5 insertion bias")
@@ -212,7 +212,7 @@ Footprint.ChromatinAssay <- function(
   for (i in seq_along(along.with = matrices)) {
     object <- SetAssayData(
       object = object,
-      slot = "positionEnrichment",
+      layer = "positionEnrichment",
       new.data = matrices[[i]],
       key = key[[i]]
     )
@@ -311,8 +311,8 @@ InsertionBias.ChromatinAssay <- function(
   insertions <- Extend(x = insertions, upstream = 3, downstream = 2)
   sequences <- as.vector(x = Biostrings::getSeq(x = genome, insertions))
   seq.freq <- table(sequences)
-  # remove sequences containing N
-  keep.seq <- !grepl(pattern = "N", x = names(x = seq.freq))
+  # remove sequences containing non-ATCG characters
+  keep.seq <- !grepl(pattern = "[^ATCGatcg]", x = names(x = seq.freq))
   insertion_hex_freq <- as.matrix(x = seq.freq[keep.seq])
   genome_freq <- Biostrings::oligonucleotideFrequency(
     x = Biostrings::getSeq(x = genome, chr.use),
@@ -326,7 +326,7 @@ InsertionBias.ChromatinAssay <- function(
   }
   insertion_hex_freq <- insertion_hex_freq[names(x = genome_freq), ]
   bias <- insertion_hex_freq / genome_freq
-  object <- SetAssayData(object = object, slot = "bias", new.data = bias)
+  object <- SetAssayData(object = object, layer = "bias", new.data = bias)
   return(object)
 }
 
@@ -411,6 +411,10 @@ FindExpectedInsertions <- function(dna.sequence, bias, verbose = TRUE) {
     # append
     x[current.pos:end.pos] <- as.numeric(x = frequencies)
     j[current.pos:end.pos] <- jj
+    
+    # remove frequencies not present in hex.key
+    frequencies <- frequencies[names(x = frequencies) %in% names(x = hex.key)]
+    
     i[current.pos:end.pos] <- as.vector(x = hex.key[names(x = frequencies)])
     # shift current position
     current.pos <- end.pos + 1
@@ -491,7 +495,7 @@ GetMotifSize <- function(
   positionEnrichment <- GetAssayData(
     object = object,
     assay = assay,
-    slot = "positionEnrichment"
+    layer = "positionEnrichment"
   )
   sizes <- c()
   for (i in features) {
@@ -545,7 +549,7 @@ RunFootprint <- function(
     )
   )
   if (compute.expected) {
-    bias <- GetAssayData(object = object, slot = "bias")
+    bias <- GetAssayData(object = object, layer = "bias")
     if (is.null(x = bias)) {
       stop("Insertion bias not computed")
     } else {
